//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1997 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Lccorr.h --- lightcurve corrector class
//
//	DESCRIPTION:
//		Lightcurve 
//
//	CHANGE HISTORY:
//		AI, 2007-05-30: 1.0.0: - first version
//
//	AUTHOR:
//		A. Ibarra, Wed May 30 16:32:18 GMT 2007
//
//	REFERENCES:
//
//
//


#ifndef LCCORR_H_
#define LCCORR_H_

#include <valarray>

#include "Dal.h"

#include "Lightcurve.h"
#include "LccorrUtils.h"

class Lccorr 
{
public:
	Lccorr(double binSize);
	~Lccorr() {delete _srcLc; delete _bkgLc;};
	
	void setTimeLimits(const Column * col);
	void setTimeLimits(double startTime, double endTime);
	void createTimeGrid();
	void setSourceData(DataSet *sourceSet);
	Lightcurve* sourceData();
	
private:
	double _startTime;
	double _endTime;
	double _binSize;
	valarray<double> _timeGrid;
	Lightcurve *_srcLc;
	Lightcurve *_bkgLc;
	LccorrUtils lcUtils;
};


#endif /*LCCORR_H_*/
