//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1997 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	LccorrUtils.cc --- lightcurve class
//
//	DESCRIPTION:
//		Lightcurve
//
//	CHANGE HISTORY:
//		AI, 2007-05-30: 1.0.0: - first version
//
//	AUTHOR:
//		A. Ibarra, Wed May 30 16:32:18 GMT 2007
//
//	REFERENCES:
//
//

#include <iomanip>
#include <cmath>

#include <errstr.h>
/* AI gcc4 porting
 *‘INT_MIN’ was not declared in this scope
 * => #include <climits>
 */
#include <climits>

#include "LccorrUtils.h"
#include "ASCregion.h"
#include "ShapeScanner.h"

inline int tint( double x )
{
   return x > INT_MIN+1 ? x < INT_MAX-1 ? (int)x : INT_MAX-1 : INT_MIN+1;
}

LccorrUtils::LccorrUtils()
{
	_betaMin = -1.0;
	_betaMax = 0.0;
	_xdspMin = -1.0;
	_xdspMax = 0.0;
	_pixels = 0.0;
	_areaPerPixel = 0.0;
}

void LccorrUtils::createGrids(const Table *etab)
{

	//_betaGrid = gridspan( etab->column("BETA_CHANNEL") );
	//_xdspGrid = gridspan( etab->column("XDSP_CHANNEL") );

	double dlt;
	double rvl;
	int    rpx;

	vector<double> betaGridLimits = getRangeInt16(etab->column("BETA_CHANNEL"));
	vector<double> xdspGridLimits = getRangeInt16(etab->column("XDSP_CHANNEL"));

    dlt = etab->column("BETA_CHANNEL")->attribute("TCDLT")->asReal();

   	rvl = etab->column("BETA_CHANNEL")->attribute("TCRVL")->asReal();
    rpx = etab->column("BETA_CHANNEL")->attribute("TCRPX")->asInt();

    //cout << "beta dlt " << dlt << endl;
    //cout << "beta rvl " << rvl << endl;
    //cout << "beta rpx " << rpx << endl;
    //cout << "beta min " << ((rvl+(betaGridLimits[0]*dlt))-(dlt/2.0)) << endl;
    //cout << "beta max " << ((rvl+(betaGridLimits[1]*dlt))-(dlt/2.0))  << endl;
    //cout << "first origin " << rvl+(dlt*betaGridLimits[0])-(dlt/2.0) << endl;
/*
    int bins =
    	tint( ceil( (((rvl+(betaGridLimits[1]*dlt))-(dlt/2.0))-((rvl+(betaGridLimits[0]*dlt))-(dlt/2.0))) /dlt ));

    cout << "nbins " << bins << endl;

    _betaGrid =
     	GridSpan(Grid(dlt,(rvl+(dlt*betaGridLimits[0]))-(dlt/2.0)),
     		Span(int (((rvl+(betaGridLimits[0]*dlt))-(dlt/2.0)))).length(bins));
*/
    int bins =
    	tint( ceil( (_betaMax-_betaMin)/dlt ) );

    //cout << "nbins " << bins << endl;

    _betaGrid =
     	GridSpan(Grid(dlt,_betaMin-(dlt/2.0)),Span(int ((_betaMin))).length(bins));


    //cout << "origin0 " << _betaGrid.origin << endl;
    //cout << "min0 " << _betaGrid.minval() << endl;
    //cout << "max0 " << _betaGrid.maxval() << endl;
    //cout << "min1 " << _betaGrid.min << endl;
    //cout << "min2 " << _betaGrid.max << endl;

 //_betaGrid = Grid (dlt,betaGridLimits[1]-betaGridLimits[0]);

     dlt = etab->column("XDSP_CHANNEL")->attribute("TCDLT")->asReal();

   	 rvl = etab->column("XDSP_CHANNEL")->attribute("TCRVL")->asReal();
     rpx = etab->column("XDSP_CHANNEL")->attribute("TCRPX")->asInt();
     //int    min = etab->column("XDSP_CHANNEL")->attribute("TLMIN")->asInt();
   	 //int    max = etab->column("XDSP_CHANNEL")->attribute("TLMAX")->asInt();

     //cout << "xdsp dlt " << dlt << endl;
     //cout << "xdsp rvl " << rvl << endl;
     //cout << "xdsp rpx " << rpx << endl;
     //cout << "xdsp min " << (rvl+(xdspGridLimits[0]*dlt))-(dlt/2.0) << endl;
     //cout << "xdsp max " << (rvl+(xdspGridLimits[1]*dlt))-(dlt/2.0)  << endl;

     _xdspGrid =
     	GridSpan(Grid(dlt,(rvl+(xdspGridLimits[0]*dlt))-(dlt/2.0)),
     		Span(int ( (rvl+(xdspGridLimits[0]*dlt))-(dlt/2.0) ),
     			 int ( (rvl+(xdspGridLimits[1]*dlt))-(dlt/2.0) ) ) );

//vector<double> betaGridLimits = getRangeInt16(etab->column("BETA_CORR"));
//vector<double> xdspGridLimits = getRangeInt16(etab->column("XDSP_CORR"));

//_xdspGrid = Grid (dlt,xdspGridLimits[1]-xdspGridLimits[0]);
}

void LccorrUtils::createTimeBins()
{

}

vector<double> LccorrUtils::getRangeInt16(const Column * col){

  // The following macro sillyness is necessary due to lack of auto-type conversion in the Dal

  vector<double> limits;
  //  std::cout << "getRange " << std::endl;
  ColumnData *cFlagData = 0;
  int16 *flagData = 0;
  cFlagData = col->data();
  flagData = cFlagData->int16Data();

  valarray<double> colVect(col->rows());

  for(unsigned long i=0; i<col->rows(); i++)
  	colVect[i]=flagData[i];

  //std::cout << std::setprecision(16)<< "min value "<<  colVect.min() << std::endl;
  //std::cout << std::setprecision(16)<< "max value " << colVect.max()  << std::endl;

  limits.push_back(colVect.min());
  limits.push_back(colVect.max());

  delete cFlagData;

  return limits;
}


vector<double> LccorrUtils::getRange(const Column * col){

  // The following macro sillyness is necessary due to lack of auto-type conversion in the Dal

  vector<double> timeLimits;
  //std::cout << "getRange " << std::endl;
  ColumnData *cFlagData = 0;
  real64 *flagData = 0;
  cFlagData = col->data();
  flagData = cFlagData->real64Data();

  valarray<double> colVect(col->rows());

  for(unsigned long i=0; i<col->rows(); i++)
  	colVect[i]=flagData[i];

  //std::cout << std::setprecision(16)<< "getRange starTime "<<  colVect.min() << std::endl;
  //std::cout << std::setprecision(16)<<"getRange endTime " << colVect.max()  << std::endl;

  timeLimits.push_back(colVect.min());
  timeLimits.push_back(colVect.max());

  delete cFlagData;

  return timeLimits;
}


void LccorrUtils::createGrid(double binSize,double startTime, double endTime)
{
	/* calculates the number of bins */
	valarray<double> timeEvents;
	//std::cout << "startTime " << startTime << std::endl;
	//std::cout << "endTime " << endTime << std::endl;

	_nBins = tint (ceil ((endTime - startTime)/binSize));
	//std::cout << "_nBins " << _nBins << std::endl;
	timeEvents.resize(_nBins,0);

	double latestBinSize = 0.0;
	/*Check if we need an extra bin at the end of the grid */
	if ( (_nBins - ((endTime - startTime)/binSize))  < 0. )
	{
		errstr <<"Error calculating the number of bins " << _nBins
			<< " " << ((endTime - startTime)/binSize)
			<<Sas::error("BinningError");
	}
	else
	{
		latestBinSize = endTime -
			(startTime +(binSize * (timeEvents.size()-1)));
	}

	/* Grid Time Spam */
	_timeGrid.resize(timeEvents.size()+1);
	for (unsigned int i = 0 ; i < _timeGrid.size()-1  ; i++)
		_timeGrid[i] = startTime + binSize * i;

	/* Latest bin */
	_timeGrid[_timeGrid.size()-1] =_timeGrid[_timeGrid.size()-2] +latestBinSize;

	/*
	for (unsigned int i = 0 ; i < _timeGrid.size() ; i++)
	 cout<< "_timeGrid " << _timeGrid[i] << endl;
	*/

}

int LccorrUtils::getNBins()
{
	return _nBins;
}

valarray<double> LccorrUtils::getTimeGrid()
{
	return _timeGrid;
}

void LccorrUtils::scan( struct Table *table)
{

   struct InASCregionTable ascReg(table);
   struct GeoRegion geoReg;
   ascReg >> geoReg;

   int col;
   unsigned count = 0;
   list<Span> spans;
   list<Span>::const_iterator itr;
   //int iy=0;
   struct RegionScanner scanner( geoReg, _betaGrid, _xdspGrid);
   struct Bounds clip(_betaGrid,_xdspGrid);
   scanner.reset(&clip);
   //cout << "finalorigin " << _betaGrid.origin << endl;
   //cout << "finalmin " << _betaGrid.minval() << endl;
   //cout << "finalmax " << _betaGrid.maxval() << endl;
   for ( scanner.reset(); scanner.next(col,spans); )
   {
      //cout << "col "<< col << endl;
   	  //cout << "_betaGrid[col] "<< _betaGrid[col] << endl;
      for ( itr = spans.begin(); itr != spans.end(); ++itr )
      {
      	 //cout << "itr->length() " <<itr->length() << endl;
      	 if ((_betaGrid.minval() <= _betaGrid[col]) &&
      	 	( _betaGrid[col] <= _betaGrid.maxval()))
      	 	{
      	 		//cout << "IN "<< _betaGrid[col] << endl;
		 		count += itr->length();
      	 	}
		  //for (iy=itr->min; iy<=itr->max; iy++) {
		  //	cout<< "_xdspGrid[iy] " << _xdspGrid[iy] << endl;
		  //}
      }
   }
   _pixels = _pixels + count;
   _areaPerPixel = _betaGrid.stride*_xdspGrid.stride;
   //cout<< "PIXELS " << count << endl;
   //cout<< "AREA " << _betaGrid.stride*_xdspGrid.stride << endl;
}

void LccorrUtils::exposureMap( struct Array *array )
{
	 struct ExposureMap emap(array);

	 if (_xdspMin == -1.0)
	 	_xdspMin = emap.xdsp.minval();
	 if (_betaMin == -1.0)
	 	_betaMin = emap.beta.minval();
	 if (_xdspMax == 0.0)
	 	_xdspMax = emap.xdsp.maxval();
	 if (_betaMax == 0.0)
	 	_betaMax = emap.beta.maxval();

	 if (emap.xdsp.minval() <   _xdspMin)
	 	_xdspMin = emap.xdsp.minval();

	 if (emap.xdsp.maxval() > _xdspMax)
	 	_xdspMax = emap.xdsp.maxval();
	 if (emap.beta.minval() < _betaMin)
	 	_betaMin = emap.beta.minval();
	 if (emap.beta.maxval() > _betaMax)
	 	_betaMax = emap.beta.maxval();
	 //cout << "emap min " << emap.beta.minval() << endl;
	 //cout << "emap max " << emap.beta.maxval() << endl;
	 //const Bounds &bnds = emap.bounds();
	 //cout << "bounds row min " <<  bnds.row.min << endl;
	 //cout << "bounds row max " <<  bnds.row.max << endl;
	 //cout << "bounds col min " <<  bnds.col.min << endl;
	 //cout << "bounds col max " <<  bnds.col.max << endl;


	 //_xdspGrid = emap.xdsp;
	 //_betaGrid = emap.beta;

}

double LccorrUtils::getArea()
{
	return 	_pixels * _areaPerPixel;
}
